@@ -2,7 +2,7 @@
 # NanoB2B-NER::NER::Arffman
 #
 # Creates ARFF files from annotated files
-# Version 1.5
+# Version 1.9
 #
 # Program by Milk
 
@@ -40,11 +40,7 @@ my $uniSub;
 my @allBuckets;
 my %fileHash;
 my %metamapHash;
-my %tokenHash;
-my %conceptHash;
-my %posHash;
-my %semHash;
-my %cuiHash;
+my %featHash;
 my %orthoHash;
 my @features;
 my $selfId = "_self";
@@ -77,15 +73,11 @@ sub new {
     @allBuckets = ();
 	%fileHash = ();
 	%metamapHash = ();
-	%tokenHash = ();
-	%conceptHash = ();
-	%posHash = ();
-	%semHash = ();
-	%cuiHash = ();
+	%featHash = ();
 	%orthoHash = ();
 	@features = ();
 
-    #bless this object
+    #bless this object - hallelujah
     bless $self, $class;
     $self->_init($params);
     @allBuckets = (1..$bucketsNum);
@@ -150,11 +142,8 @@ sub arff_file{
 	my $indexer = 0;
 	%fileHash = ();
 	%metamapHash = ();
-	%tokenHash = ();
-	%conceptHash = ();
-	%posHash = ();
-	%semHash = ();
-	%cuiHash = ();
+	%featHash = ();
+	%orthoHash = ();
 
 	#get the name of the file
 	my @n = split '/', $file;
@@ -163,10 +152,9 @@ sub arff_file{
 	$filename = lc($filename);
 
 	my $FILE;
-	open ($FILE, "$program_dir/$file") || die ("what is this '$program_dir/$filename' you speak of?\n");
+	open ($FILE, "$program_dir/$file") || die ("what is this '$program_dir/$filename' you speak of?\n$!");
 	my @fileLines = <$FILE>;
 	my @orthoLines = @fileLines;
-	#my @orthoLines = ["Hi! I'm Milk", "I have a hamster named Scott", "I like pizza"];
 	foreach my $l(@fileLines){
 		$l = lc($l);
 	}
@@ -184,7 +172,7 @@ sub arff_file{
 
 	#get the orthographic based lines
 	#my @orthoLines = <FILE>;
-	@orthoLines = retagSetOrtho(\@orthoLines);
+	@orthoLines = retagSetOrtho($file, \@orthoLines);
 
 	#$uniSub->printColorDebug("red", "TAG SET: ");
 	#$uniSub->printArr(", ", \@tagSet);
@@ -216,8 +204,11 @@ sub arff_file{
 	my $totalTokens = 0;
 	my $totalConcepts = 0;
 	foreach my $line (@cleanLines){
+		$| = 1;
+		$uniSub->printColorDebug("bright_blue", "\r" . "\t\tLine - $indexer/$totalLines");
+
 		#acquire the necessary variables
-		my $special_ID = "$indexer.ti.1";
+		my $special_ID = "1.ti.1";
 		my $meta = $metamapHash{$indexer};
 
 		#create citation first
@@ -225,67 +216,145 @@ sub arff_file{
 		my $citation = $dataStructures->getCitationWithId($special_ID);
 
 		#get tokens
-		my @tokensOut = $citation->getOrderedTokens();
-		#double array - extract the inner one
-		my @tokens = ();
-		foreach my $tt (@tokensOut){
-			my @newSet = @$tt;
-			push (@tokens, @newSet);
-		}
+		my @tokens = @{$citation->getOrderedTokens()};
 		my $tnum = @tokens;
 		$totalTokens += $tnum;
-		push (@{$tokenHash{$indexer}}, @tokens);
 
 		#get concepts
-		my @conceptsOut = $citation->getOrderedConcepts();
-		#double array - extract the inner one
-		my @concepts = ();
-		foreach my $cc (@conceptsOut){
-			my @newSet = @$cc;
-			push (@concepts, @newSet);
-		}
+		my @concepts = @{$citation->getOrderedConcepts()};
 		my $cnum = @concepts;
 		$totalConcepts += $cnum;
-		push (@{$conceptHash{$indexer}}, @concepts);
 
-		#create the ordered POS lines
-		my @posOrder = orderedTokenPOS($cleanLines[$indexer], \@tokens);
-		my $posOrderLine = join " ", @posOrder;
-		$posHash{$indexer} = $posOrderLine;
+		#grab EVERYTHING
 
-		#create the ordered semantic type lines
-		my @semantics = getConceptSets("sem", $line, \@concepts);
-		my $semanticsLine = join " ", @semantics;
-		$semHash{$indexer} = $semanticsLine;
-		
-		#create the ordered cui lines
-		my @cuis = getConceptSets("cui", $line, \@concepts);
-		my $cuiLine = join " ", @cuis;
-		$cuiHash{$indexer} = $cuiLine;
+		#get tokens first
+		foreach my $token (@tokens){
 
+			#print "token ";
+			#get text and pos
+			my $super_text = lc($token->{text});
+
+			foreach my $text (split(" ", $super_text)){		#split just in case its a compound word
+				#check if already there
+				if($featHash{$text}){
+					next;
+				}
+
+				my $pos = $token->{posTag};
+				$featHash{$text}->{pos} = $pos;
+			}
+		}
+		#get concepts second
+		foreach my $cc (@concepts){
+			foreach my $concept (@{$cc}[0]){		#two levels for some strange reason
+				#print "token ";
+				my $super_text = lc($concept->{text});
+				for my $text (split(" ", $super_text)){		#split just in case its a compound word
+					#grab original hash
+					my $hash_ref = $featHash{$text};
+					if (!$hash_ref || $hash_ref->{cui} || $hash_ref->{sem}){
+						next;
+					}
+
+					#get sem and cuis v_1
+					#$hash_ref->{cui} = $concept->{cui};
+					#$hash_ref->{sem} = $concept->{semanticTypes};
+
+					#print "$text --> ";
+					#$uniSub->printArr(", ", \@sem);
+
+					#get sem and cuis v_2
+					if($hash_ref->{cui}){
+						$hash_ref->{cui} = ($hash_ref->{cui} . $concept->{cui});
+					}else{
+						$hash_ref->{cui} = $concept->{cui};
+					}
+					
+					if($hash_ref->{sem}){
+						$hash_ref->{sem} = ($hash_ref->{sem} . $concept->{semanticTypes});
+					}else{
+						$hash_ref->{sem} = $concept->{semanticTypes};
+					}
+					
+
+					#print "$text --> ";
+					#$uniSub->printArr(", ", \@sem);
+				}
+			}
+		}
 
 		#increment to the next set
 		$indexer++;
 	}
+
+	#clean up everything
+	foreach my $key(keys %featHash){
+		my $hash_ref = $featHash{$key};
+		my %hash = %$hash_ref;
+		if($hash{cui}){
+			$hash{cui} = join(',', uniq(split(',', $hash{cui})));
+		}
+		if($hash{sem}){
+			$hash{sem} = join(',', uniq(split(',', $hash{sem})));
+		}
+	}
+
+	#print it back
+	print("\n");
+	#hash_debug();
+
 	$uniSub->printColorDebug("red", "TOKENS: $totalTokens\n");
 	$uniSub->printColorDebug("red", "CONCEPTS: $totalConcepts\n");
 
+	#exit;
+
 	
 	#######           BUCKET SORTING - TRAIN AND TEST DATA            #######
 
 	#sort the lines to buckets
-	$uniSub->printColorDebug("blue", "*Making buckets....\n");
 	my %buckets = ();
-	%buckets = sort2Buckets($totalLines, $bucketsNum);
+	if($bucketsNum > 1){
+		$uniSub->printColorDebug("blue", "*Making buckets....\n");
+		%buckets = sort2Buckets($totalLines, $bucketsNum);
+	}else{
+		push(@{$buckets{1}}, (0..($totalLines-1)));
+	}
+
+	#$uniSub->printArr(" ", $buckets{1});
+	
 
 	$uniSub->printColorDebug("blue", "*Making train and test files....\n");
 
-	zhu_li($filename, \%buckets);
+	process_file($filename, \%buckets);
 
 	$uniSub->printDebug("\n");
 
 }
 
+#print out the contents of the feature hash
+sub hash_debug{
+	foreach my $key (sort keys %featHash){
+	 	my $text = $key;
+	 	my $hash_ref = $featHash{$key};
+	 	my %hash = %$hash_ref;
+
+	 	my $pos = "";
+	 	my $cui = "";
+	 	my $sem = "";
+
+	 	if($hash{pos}){
+	 		$pos = $hash{pos};
+	 	}
+	 	if($hash{cui}){
+	 		$cui = join(',', @{$hash{cui}});
+	 	}
+	 	if($hash{sem}){
+	 		$sem = join(',', @{$hash{sem}});
+	 	}
+
+	 	print "$text - $pos - $cui - $sem\n";
+	}
+}
 
 
 ######################              LINE MANIPULATION               #####################
@@ -308,17 +377,18 @@ sub retag{
 	my @newSet = ();
 	my $charact = 0;
 	foreach my $word (@words){
+		#if word is currently in the annotation
 		if($charact){
-			if($word eq "<end>"){
+			if($word =~/<end>/){		#reached the end
 				$charact = 0;
-			}else{
+			}else{						#continue adding extension to the annotation set
 				my $charWord = "$word"."$entId"; 
 				push @newSet, $charWord;
 			}
 		}else{
-			if($word eq "<start:$id>"){
+			if($word =~/<start:[a-z0-9\-\_\s]+>/){	#start annotation set
 				$charact = 1;
-			}else{
+			}else{									#continue as normal
 				push @newSet, $word;
 			}
 		}
@@ -326,7 +396,7 @@ sub retag{
 
 	#clean up the new line
 	my $new_line = join " ", @newSet;
-	$new_line =~s/\b$entId\b//og;
+	$new_line =~s/\b$entId\b//g;
 	$new_line = $uniSub->cleanWords($new_line);
 	return $new_line;
 }
@@ -360,10 +430,11 @@ sub untag{
 	my $line = shift;
 	my $id = shift;
 
+	#remove <start:___> and <end>
 	my $input = lc($line);
 	$id = lc($id);
-	$input =~ s/ <start:$id>//og;
-	$input =~ s/ <end>//og;
+	$input =~ s/<start:[a-z0-9\-\_\s]+>//g;
+	$input =~ s/<end>//g;
 	$input = $uniSub->cleanWords($input);
 	return $input;
 }
@@ -377,6 +448,7 @@ sub untagSet{
 	my @lines = @$lines_ref;
 
 	my @clean_set = ();
+	#run untagger on each line
 	foreach my $line(@lines){
 		my $cl = untag($line, $filename);
 		push @clean_set, $cl;
@@ -394,7 +466,7 @@ sub importMetaData{
 	#create a directory to save hashtable data
 	my $META;
 	my $subdir = "_METAMAPS";
-	open($META, "<", ("$program_dir/$subdir/" . $name . "_meta")) || die ("HAHA No such thing!");
+	open($META, "<", ("$program_dir/$subdir/" . $name . "_meta")) || die ("Metamap file for " . $name . " not found! Check the _METAMAP folder in the file directory\n$!");
 	
 
 	#import metamap data from the file
@@ -404,6 +476,7 @@ sub importMetaData{
 	my $t = @newMetaLines;
 	$uniSub->printColorDebug("red", "META LINES: $t\n");
 	my $key = 0;
+	#assign each metamap line to a hash indexer
 	foreach my $mm (@newMetaLines){
 		$metamapHash{$key} = $mm;
 		$key++;
@@ -417,6 +490,7 @@ sub importMetaData{
 # input  : $input  <-- the line to retag 
 # output : $new_line <-- the retagged line
 sub retagOrtho{
+	my $filename = shift;
 	my $input = shift;
 	my $line = $input;
 
@@ -425,17 +499,17 @@ sub retagOrtho{
 	my @newSet = ();
 	my $charact = 0;
 	foreach my $word (@words){
-		if($charact){
-			if($word =~/<END>/){
+		if($charact){					#currently annotating
+			if($word =~/<END>/){			#stop annotation set
 				$charact = 0;
 			}else{
-				my $charWord = "$word"."$entId"; 
+				my $charWord = "$word"."$entId"; 	#add extension to annotation word
 				push @newSet, $charWord;
 			}
 		}else{
-			if($word =~/<START:[a-zA-Z-_]*>/g){
+			if($word =~/<START:[a-zA-Z0-9\-\_\s]+>/){	#start annotating
 				$charact = 1;
-			}else{
+			}else{										#continue as normal
 				push @newSet, $word;
 			}
 		}
@@ -443,7 +517,7 @@ sub retagOrtho{
 
 	#clean up the new line
 	my $new_line = join " ", @newSet;
-	$new_line =~s/\b$entId\b//g;
+	$new_line =~s/\s$entId\b//g;
 	$new_line = noASCIIOrtho($new_line);
 	return $new_line;
 }
@@ -451,14 +525,19 @@ sub retagOrtho{
 # input  : @lines  <-- the set of lines to retag
 # output : @tagSet <-- the retagged line
 sub retagSetOrtho{
+	my $file = shift;
 	my $lines_ref = shift;
 	my @lines = @$lines_ref;
 
+	my @n = split '/', $file;
+	my $l = @n;
+	my $filename = $n[$l - 1];
+
 	my @tagSet = ();
 	foreach my $line (@lines){
 		#retag the line
 		chomp($line);
-		my $tag_line = retagOrtho($line);
+		my $tag_line = retagOrtho($filename, $line);
 
 		#add it to the set
 		push @tagSet, $tag_line;
@@ -473,7 +552,7 @@ sub noASCIIOrtho{
 	my $line = shift;
 
 	my $new_in = $line;
-	$new_in =~ s/[^[:ascii:]]//g;
+	$new_in =~ s/[^[:ascii:]]//g;		#remove any words that do not contain ASCII characters
 	return $new_in
 }
 
@@ -489,178 +568,37 @@ sub cleanToken{
 
 	my $tokenText = $text;
 
-	#fix "# . #" tokens
-	if($tokenText =~ /\d+\s\.\s\d+/o){
-		$tokenText =~s/\s\.\s/\./og;
-	}
-
-	#fix "__ \' __" tokens
-	if($tokenText =~ /\w+\s\\\'\s\w+/o){
-		$tokenText =~s/\s\\\'\s//og;
-	}
-
-	if($tokenText =~ /[^a-zA-Z0-9]/o){
-		$tokenText = "";
-	}
+	#fix "# . #" tokens, fix "__ \' __" tokens, remove any non word based characters
+	$tokenText =~s/\s\.\s/\./og;
+	$tokenText =~s/\s\\\'\s//og;
+	$tokenText =~s/[^a-zA-Z0-9]//og;
 
 	return $tokenText;
 }
 
-#grabs the part-of-speech part of the token that's matched up with the bucket tokens
-# input  : @buktTokens     <-- the set of tokens from the specific bucket[s]
-# output : @posTokens 	   <-- the part-of-speech tokens for the bucket
-sub getTokensPOS{
-	my $bucketTokens_ref = shift;
-	my @buktTokens = @$bucketTokens_ref;
-
-	#finds the part of speech tokens
-	my @posTokens = ();
-	foreach my $token (@buktTokens){
-		my $pos = $token->{posTag};
-		push(@posTokens, $pos);
-	}
-	return @posTokens;
-}
-
-#gets the positions of the POS words from a line
-# input  : $cleanLine     <-- the line to pinpoint the POS tokens to
-#		 : @tokens        <-- the set of tokens to use are reference
-# output : @orderPOS 	   <-- the part-of-speech tokens for the bucket
-sub orderedTokenPOS{
-	my $cleanLine = shift;
-	my $tokens_ref = shift;
-	my @tokens = @$tokens_ref;
-
-	#$uniSub->printColorDebug("on_red", "TAG : $tagLine\n");
-	my @lineWords = split " ", $cleanLine;
-	my @orderPOS = ();
-
-	#make the connection between the word and the pos
-	my %text2Pos = ();
-	my @txtTokens = ();
-	foreach my $token (@tokens){
-		my $txt = $token->{text};
-		$txt = cleanToken($txt);
-		#$uniSub->printColorDebug("on_green", $txt);
-		push @txtTokens, $txt;
-		my $pos = $token->{posTag};
-		$text2Pos{$txt} = $pos;
-	}
-
-	#associate each tagged word with it
-	foreach my $word (@lineWords){
-		if($uniSub->inArr($word, \@txtTokens)){
-			my $newPos = $text2Pos{$word};
-			push @orderPOS, $newPos;
-		}else{
-			push @orderPOS, "undef";
-		}
-	}
-
-	return @orderPOS;
-}
-
-#gets the tagged parts of the concepts
-# input  : $type     	<-- what kind of concepts you want to extract [e.g. "sem", "cui"]
-#		 : $line        <-- the line to pinpoint the concepts to
-#		 : @concepts    <-- the total set of concepts to use
-# output : @conceptSet 	   <-- the set of concepts used within the line
-sub getConceptSets{
-	my $type = shift;
-	my $line = shift;
-	my $concepts_ref = shift;
-	my @concepts = @$concepts_ref;
-
-	$line = lc($line);
-
-	#assign each concept by their text name
-	my @conceptsTxt = ();
-	foreach my $concept (@concepts){
-		my $ohboi = @$concept[0];
-		my $name = lc($ohboi->{text});
-		push (@conceptsTxt, $name);
-	}
-
-	#make a clean set of text words
-	my @txtIn = split / /, $line;
-	my @clean_txt = ();
-	foreach my $word (@txtIn){
-		push @clean_txt, $word;
-	}
-
-	my $totCon = @conceptsTxt;
-	#get the set needed
-	my @conceptSet = ();
-	for(my $f = 0; $f < $totCon; $f++){
-		my @concept = @{$concepts[$f]};
-		my $txtCon = $conceptsTxt[$f];
-
-		
-		foreach my $cc (@concept){
-
-			#get the right items
-			my @items = ();
-			if($type eq "sem"){
-				my $s = $cc->{semanticTypes};
-				@items = split /,/, $s;
-			}elsif($type eq "cui"){
-				my $c = $cc->{cui};
-				@items = split /,/, $c;
-			}elsif($type eq "text"){
-				my $t = $cc->{text};
-				@items = split /,/, $t;
-			}
-			
-			#add to the concept set
-			push @conceptSet, @items;
-		}
-	}
-	return @conceptSet;
-}
-
-
 #retrieves the feature for a single word
 # input  : $word     	<-- the word to extract the features from
 #		 : $type        <-- what type of feature to extract [e.g. "pos", "sem", "cui"]
 # output : if "pos"		<-- a scalar part-of-speech value
 #		 : else			<-- an array of semantic or cui values (a single text value can have more than one of these)
 sub getFeature{
-	my $word = shift;
+	my $orig_word = shift;
 	my $type = shift;
 
-	#if retrieving pos tag
-	if($type eq "pos"){
-		#get the token and it's pos tag
-		foreach my $key (sort keys %tokenHash){
-			foreach my $token(@{$tokenHash{$key}}){
-				my $tokenTxt = $token->{text};
-				if($tokenTxt eq $word){
-					return $token->{posTag};
-				}
-			}
-		}
-		return "";
-	}elsif($type eq "sem" or $type eq "cui"){
-		#get the concept and it's cui or sem tag
-		foreach my $key (sort keys %conceptHash){
-			foreach my $concept (@{$conceptHash{$key}}){
-				my $ohboi = @$concept[0];
-				my $name = lc($ohboi->{text});
-				if($name eq $word){
-					if($type eq "sem"){
-						my @semArr = ();
-						foreach my $cc (@$concept){push(@semArr, $cc->{semanticTypes});}
-						return @semArr;
-					}elsif($type eq "cui"){
-						my @cuiArr = ();
-						foreach my $cc (@$concept){push(@cuiArr, $cc->{cui});}
-						return @cuiArr;
-					}
-				}
-			}
-		}
+	#clean up thw word
+	my $word = lc($orig_word);
+	$word =~s/[^a-zA-Z0-9\s]//;
+
+	if(!$featHash{$word}){
 		return "";
 	}
+
+	my $hash_ref = $featHash{$word};
+	my %hash = %$hash_ref;
+
+	if($hash{$type}){
+		return $hash{$type};
+	}
 	return "";
 }
 
@@ -704,11 +642,12 @@ sub sort2Buckets{
 ######################               ARFF STUFF              #####################
 	 #makes arff files for ortho, morpho, text, pos, cui, and sem attributes
 
-#zhu li!! Do the thing!!
+# Processes the file by retrieving attributes, making vectors, and splitting into buckets
+# (formally known as 'zhu li!! Do the thing!!'')
 # input  : $name     	    <-- the name of the file
 #		 : %bucketList       <-- the set of buckets with keys in them
 # output : (n arff files; n = # of buckets x (train and test) x # of features being used)
-sub zhu_li{
+sub process_file{
 	my $name = shift;
 	my $bucketList_ref = shift;
 	my %buckets = %$bucketList_ref;
@@ -722,6 +661,7 @@ sub zhu_li{
 		$attrSets{$item} = \%setOfAttr;						#gets both the vector and arff based attributes
 	}
 
+	#contain the stop words regular expressions if the parameter was defined
 	if(defined $stopwords_file){
 		$stopRegex = stop($stopwords_file);
 	}
@@ -756,6 +696,7 @@ sub zhu_li{
 
 		#go through each bucket
 		foreach my $bucket (sort keys %buckets){
+			#if wcs parameter defined - skip these buckets until the specified bucket and feature is reached
 			if(($wcs) && (!$wcs_found) && ($bucket != $wcs_bucket)){
 				print("\t**SKIP**\n");
 				next;
@@ -763,7 +704,13 @@ sub zhu_li{
 				$wcs_found = 1;
 			}
 
-			my @range = $uniSub->bully($bucketsNum, $bucket);
+			#make train-test bucket indexes
+			my @range = ();
+			if($bucketsNum > 1){
+				@range = $uniSub->bully($bucketsNum, $bucket);
+			}else{
+				@range = (1);
+			}
 
 			$uniSub->printColorDebug("on_green", "BUCKET #$bucket");
 			#retrieve the vector attributes to use
@@ -802,16 +749,20 @@ sub zhu_li{
 			$uniSub->printDebug("\n");
 
 			### TEST ###
-			$uniSub->printColorDebug("bold magenta", "\ttesting...\n");
-			#retrieve the lines to use
-			my @lineSetTest = ();
-			my @bucketSetTest = ();
-			push(@bucketSetTest, @{$buckets{$bucket}});
-			foreach my $key (@bucketSetTest){push(@lineSetTest, $orthoHash{$key});}
+			my @vectorSetTest = ();
+			if($bucketsNum > 1){		#skip this if only 1 bucket being used (train bucket)
+				$uniSub->printColorDebug("bold magenta", "\ttesting...\n");
+				#retrieve the lines to use
+				my @lineSetTest = ();
+				my @bucketSetTest = ();
+				push(@bucketSetTest, @{$buckets{$bucket}});
+				foreach my $key (@bucketSetTest){push(@lineSetTest, $orthoHash{$key});}
 
-			#make the vector
-			my @vectorSetTest = vectorMaker(\@lineSetTest, \@curFeatSet, \%vecAttrSet);
-			$uniSub->printDebug("\n");
+				#make the vector
+				@vectorSetTest = vectorMaker(\@lineSetTest, \@curFeatSet, \%vecAttrSet);
+				$uniSub->printDebug("\n");
+			}
+			
 
 			### ARFF ###
 			#retrieve the arff attributes to use
@@ -839,17 +790,20 @@ sub zhu_li{
 				}
 			}
 
-
+			#create the arff files for the test and train features
 			$uniSub->printColorDebug("bright_yellow", "\tmaking arff files...\n");
 			$uniSub->printColorDebug("bright_red", "\t\tARFF TRAIN\n");
 			createARFF($name, $bucket, $abbrev, "train", \@arffAttrSet, \@vectorSetTrain);
-			$uniSub->printColorDebug("bright_red", "\t\tARFF TEST\n");
-			createARFF($name, $bucket, $abbrev, "test", \@arffAttrSet, \@vectorSetTest);
+			if($bucketsNum > 1){
+				$uniSub->printColorDebug("bright_red", "\t\tARFF TEST\n");
+				createARFF($name, $bucket, $abbrev, "test", \@arffAttrSet, \@vectorSetTest);
+			}
+			
 		}
 	}
-
 }
 
+
 #create the arff file
 # input  : $name     	    <-- the name of the file
 #		 : $bucket   		<-- the index of the bucket you're testing [e.g. bucket #1]
@@ -875,12 +829,12 @@ sub createARFF{
 		my $arffdir = $name . "_ARFF";
 		my $featdir = "_$abbrev";
 		make_path("$program_dir/$subdir/$arffdir/$featdir/$typeDir");
-		open($ARFF, ">", ("$program_dir/$subdir/$arffdir/$featdir/$typeDir/" . $name . "_$type-" . $bucket .".arff")) || die ("OMG?!?!");
+		open($ARFF, ">", ("$program_dir/$subdir/$arffdir/$featdir/$typeDir/" . $name . "_$type-" . $bucket .".arff")) || die ("Cannot write ARFF file to the directory! Check permissions!\n$!");
 	}else{
 		my $arffdir = $name . "_ARFF";
 		my $featdir = "_$abbrev";
 		make_path("$arffdir/$featdir/$typeDir");
-		open($ARFF, ">", ("$arffdir/$featdir/$typeDir/" . $name . "_$type-" . $bucket .".arff")) || die ("What?!?!");
+		open($ARFF, ">", ("$arffdir/$featdir/$typeDir/" . $name . "_$type-" . $bucket .".arff")) || die ("Cannot write ARFF file to the directory! Check permissions!\n$!");
 	}
 
 	#get the attr and vector set
@@ -890,7 +844,7 @@ sub createARFF{
 	#get format for the file
 	my $relation = "\@RELATION $name";	
 	my @printAttr = makeAttrData(\@attrARFFSet);	
-	my $entity = "\@ATTRIBUTE Entity {Yes, No}";	#set if the entity word or not
+	my $entity = "\@ATTRIBUTE Entity {No, Yes}";	#set if the entity word or not
 	my $data = "\@DATA";
 
 	#print everything to the file
@@ -925,10 +879,12 @@ sub vectorMaker{
 
 	for(my $l = 0; $l < $setLen; $l++){
 		my $line = $txtLineSet[$l];
+		#$uniSub->printColorDebug("on_red", $line);
 		my @words = split(' ', $line);
 		#$uniSub->printArr(", ", \@words);
 		#print "\n";
 		my $wordLen = @words;
+
 		#go through each word
 		for(my $a = 0; $a < $wordLen; $a++){
 
@@ -945,11 +901,14 @@ sub vectorMaker{
 			my $a2 = $a + 1;
 			$uniSub->printDebug("\r" . "\t\tLine - $l2/$setLen ------ Word - $a2/$wordLen  ----  ");
 			
+			#cut-off a word in the display if it is too long 
+			#(longer than 8 characters)
 			my $smlword = substr($word, 0, 8);
 			if(length($word) > 8){
 				$smlword .= "...";
 			}
 			
+			#distinguish entity words from normal words
 			if($word =~/$entId/o){
 				$uniSub->printColorDebug("red", "$smlword!                ");
 			}else{
@@ -961,11 +920,19 @@ sub vectorMaker{
 			#$uniSub->printColorDebug("red", "\n\t\t$word - $ncui\n");
 
 			#check if it's a stopword
-			if(($stopwords_file and $word=~/$stopRegex/o) || ($is_cui and $word_cuis[0] eq "") || ($word eq "." || $word eq ",")){
-				#$uniSub->printColorDebug("on_red", "\t\tSKIP!");
+			if(($stopwords_file and $word=~/$stopRegex/o) || ($is_cui and $word_cuis[0] eq "")){
+				#$uniSub->printColorDebug("on_red", "\t$word\tSKIP!");
+				if(!($word =~/[a-zA-Z0-9\_\-]+$entId/)){
+					next;
+				}
+			}
+
+			#if a weird character word - skip
+			if((length($word) eq 1) and ($word =~/[^a-zA-Z0-9\_\-]/)){
 				next;
 			}
 
+			#get the word before and after the current word
 			if($a > 0){$prevWord = $words[$a - 1];}
 			if($a < ($wordLen - 1)){$nextWord = $words[$a + 1];}
 
@@ -1000,7 +967,7 @@ sub vectorMaker{
 
 			#check if the word is an entity or not
 			#$uniSub->printColorDebug("red", "\n$wordOrig\n");
-			$vec .= (($wordOrig =~/\b[\S]+(_e)\b/) ? "Yes " : "No ");
+			$vec .= (($wordOrig =~/\b[\S]+($entId)\b/) ? "Yes " : "No ");
 
 			#close it if using sparse matrix
 			if($sparse_matrix){
@@ -1009,7 +976,6 @@ sub vectorMaker{
 
 			#finally add the word back and add the entire vector to the set
 			$vec .= "\%$word";
-
 			if($word ne ""){
 				push(@setVectors, $vec);
 			}
@@ -1030,7 +996,7 @@ sub orthoVec{
 	my $addon = "";
 
 	#check if first letter capital
-	$addon = ($word =~ /\b([A-Z])\w+\b/og ? 1 : 0);
+	$addon = ($word =~ /\b([A-Z])\w+\b/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	#check if a single letter word
@@ -1038,23 +1004,23 @@ sub orthoVec{
 	$strVec .= "$addon, ";
 
 	#check if all capital letters
-	$addon = ($word =~ /\b[A-Z]+\b/og ? 1 : 0);
+	$addon = ($word =~ /\b[A-Z]+\b/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	#check if contains a digit
-	$addon = ($word =~ /[0-9]+/og ? 1 : 0);
+	$addon = ($word =~ /[0-9]+/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	#check if all digits
-	$addon = ($word =~ /\b[0-9]+\b/og ? 1 : 0);
+	$addon = ($word =~ /\b[0-9]+\b/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	#check if contains a hyphen
-	$addon = ($word =~ /-/og ? 1 : 0);
+	$addon = ($word =~ /-/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	#check if contains punctuation
-	$addon = ($word =~ /[^a-zA-Z0-9\s]/og ? 1 : 0);
+	$addon = ($word =~ /[^a-zA-Z0-9\s]/o ? 1 : 0);
 	$strVec .= "$addon, ";
 
 	return $strVec;
@@ -1071,9 +1037,11 @@ sub morphVec{
 
 	my $strVec = "";
 
+	#grab the first # characters and the last # characters
 	my $preWord = substr($word, 0, $prefix);
 	my $sufWord = substr($word, -$suffix);
 
+	#compare and build a binary vector
 	foreach my $a (@attrs){
 		if($a eq $preWord){
 			$strVec .= "1, ";
@@ -1239,6 +1207,7 @@ sub convert2Sparse{
 	for(my $c=0;$c<$numVals;$c++){
 		my $curVal = $vals[$c];
 
+		#if a non-zero value is found at the index - add it to the final
 		if(($curVal eq "1")){
 			$sparse_vec .= "$c $curVal, ";
 			#$uniSub->printColorDebug("red", "$c $curVal, ");
@@ -1263,9 +1232,10 @@ sub grabAttr{
 	my %buckets = %$buckets_ref;
 
 	my %vecARFFattr = ();
+	#no importing of attributes is needed for ortho since they all check the same features
 	if($feature eq "ortho"){
 		my @vecSet = ();
-		my @arffSet = ("first_letter_capital", 
+		my @arffSet = ("first_letter_capital", 		#output for the ARFF file
 						"single_character",
 						"all_capital",
 						"has_digit",
@@ -1275,13 +1245,20 @@ sub grabAttr{
 		$vecARFFattr{vector} = \@vecSet;
 		$vecARFFattr{arff} = \@arffSet;
 		return %vecARFFattr;
-	}elsif($feature eq "morph"){	
+	}
+	#the morphological attributes look at the prefix and suffix of a word not the adjacencies
+	elsif($feature eq "morph"){	
 		my %bucketAttr = ();
 		my %bucketAttrARFF = ();
 
 		#get the attributes for each bucket
 		foreach my $testBucket (@allBuckets){
-			my @range = $uniSub->bully($bucketsNum, $testBucket);
+			my @range = ();
+			if($bucketsNum > 1){
+				@range = $uniSub->bully($bucketsNum, $testBucket);
+			}else{
+				@range = (1);
+			}
 			$uniSub->printDebug("\t\t$name BUCKET #$testBucket/$feature MORPHO attributes...\n");
 			
 			#get attributes [ unique and deluxe ]
@@ -1299,18 +1276,24 @@ sub grabAttr{
 		$vecARFFattr{arff} = \%bucketAttrARFF;
 
 		return %vecARFFattr;
-	}else{
+	}
+	#text, part-of-speech, semantics, cui attributes
+	else{
 		my %bucketAttr = ();
 		my %bucketAttrARFF = ();
 
 		#get the attributes for each bucket
 		foreach my $testBucket (@allBuckets){
-			my @range = $uniSub->bully($bucketsNum, $testBucket);
+			my @range = ();
+			if($bucketsNum > 1){
+				@range = $uniSub->bully($bucketsNum, $testBucket);
+			}else{
+				@range = (1);
+			}
 			$uniSub->printDebug("\t\t$name BUCKET #$testBucket/$feature attributes...\n");
 			
 			#get attributes [ unique and deluxe ]
 			my @attr = getRangeAttributes($feature, \@range, \%buckets);
-			@attr = uniq(@attr);						#make unique forms
 			$bucketAttr{$testBucket} = \@attr;
 
 			my @attrARFF = getAttrDelux($feature, \@attr);
@@ -1325,26 +1308,6 @@ sub grabAttr{
 	}
 }
 
-
-
-#makes an array with unique elements
-# input  : @orig_arr <-- the original array w/ repeats
-# output : @new_arr  <-- same array but w/o repeats
-sub makeUniq{
-	my $orig_arr_ref = shift;
-	my @orig_arr = @$orig_arr_ref;
-
-	my @new_arr = ();
-	foreach my $t (@orig_arr){
-		unless($uniSub->inArr($t, \@new_arr) or $t =~/\s+/o or $t =~/\b$entId\b/o or length($t) == 0){
-			push @new_arr, $t;
-		}
-	}
-	@new_arr = grep { $_ ne '' } @new_arr;
-	return @new_arr;
-}
-
-
 #returns the attribute values of a range of buckets
 # input  : $type     	    <-- the feature type [e.g. ortho, morph, text]
 #		 : @bucketRange     <-- the range of the buckets to use [e.g.(1-8,10) out of 10 buckets; use "$uniSub->bully" subroutine in UniversalRoutines.pm]
@@ -1366,74 +1329,51 @@ sub getRangeAttributes{
 		}
 	}
 
-	#get the tokens for each associated key
-	my @bucketTokens = ();
+	#get the lines
+	my @bucketLines = ();
 	foreach my $key (@keyRing){
-		push @bucketTokens, @{$tokenHash{$key}};
+		push(@bucketLines, $fileHash{$key});
 	}
 
-	#get the concepts for each associated key
-	my @bucketConcepts = ();
-	if($type eq "sem"){
-		foreach my $key (@keyRing){
-			push @bucketConcepts, $semHash{$key};
-		}
-	}elsif($type eq "cui"){
-		foreach my $key (@keyRing){
-			push @bucketConcepts, $cuiHash{$key};
-		}
-	}
-
-
-	#get particular value from the tokens and concepts
+	#gather the attributes based on the words in the lines
 	my @attributes = ();
-	if($type eq "text" or $type eq "pos"){					#get the text attributes
-		my @tokenWords = ();
-		foreach my $token(@bucketTokens){
-			my $tokenText = $token->{text};
+	foreach my $line (@bucketLines){	#in each line
+		foreach my $o_word (split(' ', $line)){		#each word
+			my $word = $o_word;
+			$word =~s/$entId//;							#remove the annotation marker
+			$word =~s/<START:[a-zA-Z0-9\-\_\s]+>//;		#remove the <START:___> tag
+			$word =~s/<END>//;							#remove the <END> tag
+			$word =~s/[\:\.\-]//;						#remove any : . - characters
 
-			#add to the tokens
-			if($tokenText =~ /\w+\s\w+/o){
-				my @tokenText2 = split(" ", $tokenText);
-				push @tokenWords, @tokenText2;
-			}elsif($tokenText ne "." and $tokenText ne "-" and !($tokenText =~ /[^a-zA-Z0-9]/)){
-				push @tokenWords, $tokenText;
+			#if word is empty - skip
+			if($word eq ""){
+				next;
 			}
 
-			#clean up the text
-			foreach my $toky(@tokenWords){
-				$toky = cleanToken($toky);
+			#if only looking for text attributes - use the actual word
+			if($type eq "text"){
+				push(@attributes, $word);
+				next;
 			}
 
+			#skip if dne
+			if(!$featHash{$word}){
+				next;
+			}
+			#grab attributes if it does
+			my $hash_ref = $featHash{$word};
+			my %hash = %$hash_ref;
+
+			if($hash{$type}){
+				push(@attributes, split(',', $hash{$type}));
+			}
 		}
-		#gets the tokens for the attributes and vector analysis
-		if($type eq "text"){
-			@attributes = @tokenWords;
-		}elsif($type eq "pos"){
-			@attributes = getTokensPOS(\@bucketTokens, \@tokenWords, \@keyRing);
-		}
-		
 	}
-	#get the concept-based attributes
-	elsif($type eq "sem" or $type eq "cui"){					
-		my @conWords = ();
-		foreach my $conFeat(@bucketConcepts){
-			my @conLine = split / /, $conFeat;
-			push @conWords, @conLine;
-		}
-		@attributes = uniq (@conWords);
-
-		#add a semantic label for differentiation
-		if($type eq "sem"){
-			foreach my $a (@attributes){$a = uc($a);}
-		}
-
-	}
-	#my $a = @attributes;
-	#$uniSub->printColorDebug("red", "$type ATTR: #$a\n");
-	#printArr("\n", @attributes);
 
+	@attributes = uniq(@attributes);
+	#$uniSub->printArr(",", \@attributes);
 	return @attributes;
+
 }
 
 #makes the arff version attributes - makes a copy of each attribute but with "_self" at the end
@@ -1451,6 +1391,8 @@ sub getAttrDelux{
 		#check if certain type of feature
 		if($f eq "pos"){
 			$word = ($word . "_POS");
+		}elsif($f eq "sem"){
+			$word = uc $word;
 		}
 		$word =~s/$entId//g;
 
@@ -1464,57 +1406,6 @@ sub getAttrDelux{
 	return @attrDelux;
 }
 
-#returns the lines from a range of buckets
-# input  : $type     	    <-- the feature type [e.g. ortho, morph, text]
-#		 : @bucketRange     <-- the range of the buckets to use [e.g.(1-8,10) out of 10 buckets; use "$uniSub->bully" subroutine in UniversalRoutines.pm]
-#		 : %buckets 		<-- the bucket key set
-# output : @bucketLines		<-- the lines for the specific type and bucket keys based on the range
-sub getRangeLines{
-	my $type = shift;
-	my $bucketRange_ref = shift;
-	my $buckets_ref = shift;
-	my @bucketRange = @$bucketRange_ref;
-	my %buckets = %$buckets_ref;
-
-	#collect all the necessary keys
-	my @keyRing = ();
-	foreach my $bucket (sort { $a <=> $b } keys %buckets){
-		my @bucKeys = @{$buckets{$bucket}};
-		if($uniSub->inArr($bucket, \@bucketRange)){
-			push @keyRing, @bucKeys;
-		}
-	}
-
-	my @bucketLines = ();
-	#get the lines for each associated key
-	if($type eq "text"){
-		#[line based]
-		foreach my $key (@keyRing){
-			my $line = $fileHash{$key};
-			push @bucketLines, $line;
-		}
-	}
-	elsif($type eq "pos"){
-		foreach my $key (@keyRing){
-			my $line = $posHash{$key};
-			push @bucketLines, $line;
-		}
-	}
-	elsif($type eq "sem"){
-		foreach my $key (@keyRing){
-			my $line = $semHash{$key};
-			push @bucketLines, $line;
-		}
-	}
-	elsif($type eq "cui"){
-		foreach my $key (@keyRing){
-			my $line = $cuiHash{$key};
-			push @bucketLines, $line;
-		}
-	}
-
-	return @bucketLines;
-}
 #looks at the prefix # and suffix # and returns a substring of each word found in the bucket text set
 # input  : @bucketRange     <-- the range of the buckets to use [e.g.(1-8,10) out of 10 buckets; use "$uniSub->bully" subroutine in UniversalRoutines.pm]
 #		 : %buckets 		<-- the bucket key set
@@ -1578,7 +1469,8 @@ sub makeAttrData{
 	return @attributes;
 }
 
-##new stoplist function
+##new stoplist function - by Dr. McInnes
+#generates a regex expression searching for the stop list words
 sub stop { 
  
     my $stopfile = shift; 
@@ -1587,7 +1479,7 @@ sub stop {
     my $stop_mode = "AND";
 
     open ( STP, $stopfile ) ||
-        die ("Couldn't open the stoplist file $stopfile\n");
+        die ("Couldn't open the stoplist file $stopfile\n$!");
     
     while ( <STP> ) {
 	chomp; 
@@ -1643,4 +1535,4 @@ sub stop {
     return $stop_regex; 
 }
 
-1;
\ No newline at end of file
+1;
