@@ -108,6 +108,7 @@ sub avg_file{
 	my $self = shift;
 	my $name = shift;
 
+	#get the averaged weka data, individual weka data, and majority sense baseline
 	my %avg = averageWekaData($name);
 	my %ind = individualWekaData($name);
 	my $msb = getMSB($name);
@@ -117,7 +118,7 @@ sub avg_file{
 	make_path($direct);		
 	my $file = "$direct/$name" . "_weka-results";		
 	my $WEKA_SAVE;
-	open ($WEKA_SAVE, ">", "$file") || die ("Aw man! $!");
+	open ($WEKA_SAVE, ">", "$file") || die ("Weka file $file could not be made in the _WEKAS directory! \n$!");
 
 	#print the name
 	$uniSub->print2File($WEKA_SAVE, "ENTITY NAME\t\t: $name"); 
@@ -143,6 +144,7 @@ sub avg_file{
 		for(my $a = 0; $a < $arrLen; $a++){
 			my $wekaThing = $wekaAttr[$a];
 			my $entry = $arr[$a];
+			#try to align the values based on the length
 			my $tab = "";
 			if(length($wekaThing) > 7){
 				$tab = "\t";
@@ -208,7 +210,7 @@ sub averageWekaData{
 
 			#import the wekaman
 			my $file = "$program_dir/_WEKAS/$weka_dir/$name" . "_WEKA_DATA/$item/$name" . "_accuracy_$bucket";
-			open (WEKA, "$file") || die ("WHY NO FILE - $file?!");
+			open (WEKA, "$file") || die ("Unable to import $file from _WEKAS directory\n $!");
 
 			#get lines
 			my @lines = <WEKA>;
@@ -217,8 +219,9 @@ sub averageWekaData{
 
 			#$uniSub->printColorDebug("on_red", "YOUR FACE!");
 
+			#file doesn't have any contents for some reason
 			if($len == 0){
-				$uniSub->printColorDebug("red", "NOTHING FOUND FOR $file!\n");
+				$uniSub->printColorDebug("red", "**ERROR: NOTHING FOUND FOR $file!**\n");
 				my $n = @wekaAttr;
 				my @values = (0) x $n;
 				push(@{$data{$bucket}}, @values);
@@ -254,7 +257,7 @@ sub averageWekaData{
 			my $wekaLen = @wekaSet;
 			for(my $e = 0; $e < $wekaLen; $e++){
 				my $entry = $wekaSet[$e];
-				if($entry ne "NaN" and $entry ne "?"){
+				if($entry ne "NaN" and $entry ne "?"){		#if value is unknown (from WEKA)
 					$averages[$e] += $entry;
 				}
 			}
@@ -267,11 +270,8 @@ sub averageWekaData{
 			$tea /= $bucketsNum;
 		}
 
-		#ta-da averages
+		#add the averages for the specific feature
 		push(@{$featAvg{$item}}, @averages);
-
-		#hello I am here(>'_')> hug me~
-		#ok then <(^_^<) ~
 	}
 	#exit;
 	return %featAvg;
@@ -284,22 +284,23 @@ sub individualWekaData{
 	my $name = shift;
 	my %featData = ();
 
-	foreach my $item (@sets){
+	foreach my $item (@sets){		#for each feature specified
 		my @data = ();
 
-		foreach my $bucket(@allBuckets){
+		foreach my $bucket(@allBuckets){	#for each bucket in the set
 
 			#import the wekaman
 			my $file = "$program_dir/_WEKAS/$weka_dir/$name" . "_WEKA_DATA/$item/$name" . "_accuracy_$bucket";
-			open (WEKA, "$file") || die ("WHY NO FILE - $file?!");
+			open (WEKA, "$file") || die ("Unable to import $file from _WEKAS directory $!");
 
 			#get lines
 			my @lines = <WEKA>;
 			foreach my $line(@lines){chomp($line)};
 			my $len = @lines;
 
+			#if the contents of the file are empty for some reason
 			if($len == 0){
-				$uniSub->printColorDebug("red", "NOTHING FOUND FOR $file!\n");
+				$uniSub->printColorDebug("red", "**ERROR: NOTHING FOUND FOR $file!**\n");
 				my $n = @wekaAttr;
 				my @values = (0) x $n;
 				my $valueLine = join(" ", @values);
@@ -318,7 +319,7 @@ sub individualWekaData{
 			my $weightIndex = $uniSub->getIndexofLine($weightWord, \@result);
 			my $weightLine = $result[$weightIndex];
 
-			#split it uuup
+			#split it up
 			my @values = split /\s+/, $weightLine;
 			my $valLen = @values;
 			$valLen -= 2;
@@ -347,8 +348,8 @@ sub getMSB{
 	$filename = lc($filename);
 
 	#import the data from the file
-	if($program_dir ne ""){open (FILE, "$program_dir/$file") || die ("what is this '$program_dir/$file' you speak of?\n");}
-	else{open (FILE, "$file") || die ("what is this 'file' you speak of?\n");}
+	if($program_dir ne ""){open (FILE, "$program_dir/$file") || die ("what is this '$program_dir/$file' you speak of?\n$!");}
+	else{open (FILE, "$file") || die ("what is this '$file' you speak of? $!\n");}
 	my @fileLines = <FILE>;
 	foreach my $l(@fileLines){
 		$l = lc($l);
@@ -364,7 +365,6 @@ sub getMSB{
 # counts the # of no instances / # of all instances
 # input  : @set   <-- set of lines to read and count
 # output : $msb   <-- # of no instances / # of all instances
-
 sub countInst{
 	my $set_ref = shift;
 	my @set = @$set_ref;
@@ -376,14 +376,16 @@ sub countInst{
 		my @words = split(" ", $line);
 		foreach my $word(@words){
 			#print($word . "\n");
+			#if the word doesn't have a entity tag
 			if(!($word =~ /([\s\S]*($entId)$)/)){
-				$no_inst++;
+				$no_inst++;		#count the words not annotated
 			}
 
-			$all_inst++;
+			$all_inst++;		#count all the words
 		}
 	}
 
+	#return the ratio
 	return ($no_inst / $all_inst);
 }
 
@@ -407,17 +409,17 @@ sub retag{
 	my @newSet = ();
 	my $charact = 0;
 	foreach my $word (@words){
-		if($charact){
-			if($word eq "<end>"){
+		if($charact){				#if currently annotating
+			if($word eq "<end>"){		#stop tagging annotations
 				$charact = 0;
-			}else{
+			}else{						#add the entity tagger to the words
 				my $charWord = "$word"."$entId"; 
 				push @newSet, $charWord;
 			}
 		}else{
-			if($word eq "<start:$id>"){
+			if($word eq "<start:$id>"){		#start annotating
 				$charact = 1;
-			}else{
+			}else{							#add normal word
 				push @newSet, $word;
 			}
 		}
