@@ -100,7 +100,7 @@ sub meta_file{
 
 	#import the data from the file
 	my $FILE;
-	open ($FILE, "$program_dir/$file") || die ("what is this 'dir/file' you speak of?\n");
+	open ($FILE, "$program_dir/$file") || die ("what is this '$program_dir/$file' you speak of?\n$!");
 	my @fileLines = <$FILE>;
 	foreach my $l(@fileLines){
 		$l = lc($l);
@@ -239,9 +239,9 @@ sub exportMetaData{
 	if($program_dir ne ""){
 		my $subdir = "_METAMAPS";
 		$uniSub->make_path("$program_dir/$subdir");
-		open($META, ">", ("$program_dir/$subdir/" . $name . "_meta")) || die ("Um...");
+		open($META, ">", ("$program_dir/$subdir/" . $name . "_meta")) || die ("Cannot find $name metamap file! Check _METAMAPS $!");
 	}else{
-		open($META, ">", ($name . "_meta")) || die ("Agh!");
+		open($META, ">", ($name . "_meta")) || die ("Cannot find $name metamap file in local directory! $!");
 	}
 
 	#print metamap data to the file
