@@ -52,7 +52,7 @@ sub new {
 
 
 #excludes a value from a number array
-# input  : $e 	  <-- the number of elements in the array (5 = 1,2,3,4,5)
+# input  : $e 	  <-- the range of elements in the array from 1-$e ($e=5 => 1,2,3,4,5)
 #		   $bully <-- the number to exclude from the set (3 => 1,2,4,5)
 # output : @kids  <-- the number set
 sub bully{
@@ -75,8 +75,7 @@ sub cleanWords{
 	$input = lc($input);
 	$input =~ s/[^a-zA-z0-9\:\.\s<>&#;\*\/]/ /og; 	#get rid of non-ascii
 	#$input =~ s/([0-9]+(\.[0-9]*)?)-[0-9]+(\.[0-9]*)?/RANGE/g;		#get rid of range num (#-#)
-	$input =~ s/\b[0-9]+\.?[0-9]+\b/NUM/og;			#get rid of normal num (#.#)
-	$input =~ s/\b[0-9]+\b/NUM/og;					#get rid of normal num (#.#)
+	$input =~ s/[0-9]+(.[0-9]+)?/NUM/og;			#get rid of normal num (#.#)
 	$input =~ s/\s?=\s?/eq/og;						#get rid of = 
 	$input =~ s/<Node id.*?\/>//og;					#get rid of <NODE id=##/> 
 	$input =~ s/[\*\/]//og;							#get rid of * and /
@@ -115,7 +114,7 @@ sub getIndexofLine{
 	my $len = @lines;
 	for(my $a = 0; $a < $len; $a++){
 		my $line = $lines[$a];
-		if ($line =~ /($keyword)/){
+		if ($line =~ /($keyword)/){			#regex checking if string contains keyword
 			return $a;
 		}
 	}	
@@ -142,19 +141,19 @@ sub inArr{
 
 #prints to a file called debug
 # input  : $output 	 <-- the text to output to the debug file
-# output : --
+# output : a local file named 'debug'
 sub print2DebugFile{
 	my $self = shift;
 	my $output = shift;
 
-	open(DEBUG, ">>", "debug") || die "NOOOOOO DEBUG NOOOOOO!!! >w<";
+	open(DEBUG, ">>", "debug") || die "**ERROR: Unable to create debug file!**\n$!";
 	print2File(<DEBUG>, $output);
 }
 
 #prints to a file w/ line skip
 # input  : $file 	 <-- the file to print to (must already be opened!)
 #		   @array    <-- the text to print to the file (includes next line)
-# output : --
+# output : the file with the text printed to it (with a return character)
 sub print2File{
 	my $self = shift;
 	my $file = shift;
@@ -166,7 +165,7 @@ sub print2File{
 #prints to a file as is
 # input  : $file 	 <-- the file to print to (must already be opened!)
 #		   @array    <-- the text to print to the file
-# output : --
+# output : the file with the text printed to it without a return character
 sub print2FileNoLine{
 	my $self = shift;
 	my $file = shift;
@@ -191,7 +190,7 @@ sub printArr{
 
 
 #prints input with color
-# input  : $color 	<-- color to print the text in
+# input  : $color 	<-- color to print the text in (ex. 'red', 'bold blue', 'on_green')
 #		   $text    <-- the text to print
 # output : --
 sub printColor{
@@ -204,7 +203,7 @@ sub printColor{
 }
 
 #prints input with color for debug mode only
-# input  : $color 	<-- color to print the text in
+# input  : $color 	<-- color to print the text in (ex. 'magenta', 'bright_cyan', 'on_bright_yellow')
 #		   $text    <-- the text to print
 # output : --
 sub printColorDebug{
@@ -243,9 +242,11 @@ sub setInArr{
 	my @arr1 = @$arr1_ref;
 	my @arr2 = @$arr2_ref;
 
+	#combine the arrays as a string
 	my $str1 = join " ", @arr1;
 	my $str2 = join " ", @arr2;
 
+	#check for the substring of arr2 in arr1
 	if(index($str2, $str1) != -1){
 		return 1;
 	}else{
@@ -278,6 +279,7 @@ sub wordIndex{
 
 	my @curWords = ();
 	my @index_set = ();
+	#for each word in the set count it's frequency
 	foreach my $word (@words){
 		my $index = wordCount($word, @curWords) + 1;
 		push(@curWords, $word);
